# frozen_string_literal: true

module Gitlab
  module Mercurial
    class HgitalyRepository < HgGitRepository
      extend ::Gitlab::Utils::Override

      def initialize(*args, hpd_original: nil)
        super(*args)
        @hpd_original = hpd_original
      end

      def hpd_original
        @hpd_original.nil? ? self : @hpd_original
      end

      def hg_sha_map
        @hpd_original.nil? ? super : @hpd_original.hg_sha_map
      end

      def git_sha_map
        @hpd_original.nil? ? super : @hpd_original.git_sha_map
      end

      def cached_exists?
        @hpd_original.nil? ? super : @hpd_original.cached_exists?
      end

      override :archive_sha
      def archive_sha(sha)
        sha
      end

      # Compatibility scaffolding for uniformity with HgGitRepository
      # the SHAs we're handling in this class are directly Mercurial ones.
      # Of course, eventually we won't have HGitRepository around, and
      # we won't need the callers of this in Git::Commit and elsewhere
      def hgsha_from_sha(sha)
        sha
      end

      # Compatibility scaffolding: see hgsha_from_sha
      def sha_from_hgsha(hgsha)
        hgsha
      end

      # Compatibility scaffolding: see Repository#gitsha_from_sha
      def gitsha_from_sha(sha)
        return unless cached_exists?

        git_sha_map[sha]
      end

      # Compatibility scaffolding: see Repository#gitsha_from_sha
      def sha_from_gitsha(gitsha)
        return unless cached_exists?

        hg_sha_map[gitsha]
      end

      # Compatibility scaffolding, similar to hgsha_from_sha
      #
      # the difference here is that this method does not promise
      # to return a Mercurial SHA
      def commit_id_to_display_id(commit_id)
        commit_id
      end

      # See commit_id_to_display_id
      def commit_id_from_display_id(display_id)
        display_id
      end

      # Clone current repository with a VCS qualified value for `storage`
      def vcs_storage_clone(vcs)
        self.class.new(
          Gitlab::GitalyClient.vcs_qualified_storage(@storage, vcs),
          @relative_path,
          @gl_repository,
          @gl_project_path,
          hpd_original: self
        )
      end

      override :hg_env_for_write
      def hg_env_for_write(*args, **kwargs)
        res = super
        res['HEPTAPOD_HG_NATIVE'] = 'yes'
        res
      end

      def list_last_commits_for_tree(sha, path, **kwargs)
        commits = super(gitsha_from_sha(sha), path, **kwargs)
        commits.each_value { |c| c.hpd_back_convert!(self) }
        commits
      end

      def merge_base(*commit_ids)
        git_commit_ids = commit_ids.map { |sha| gitsha_from_sha(sha) }
        base_sha = super(*git_commit_ids)
        sha_from_gitsha(base_sha) unless base_sha.nil?
      end

      # convert ids on the fly, for this through gitaly_commit_client
      # that actually invokes the DiffService (not provided yet by HGitaly).
      def diff_stats(left_id, right_id)
        # incoming "ids" can be Git::EMPTY_TREE_ID or perhaps even
        # non-sha revisions
        super(fallbacking_gitsha_from_sha(left_id),
              fallbacking_gitsha_from_sha(right_id))
      end

      def gitaly_ref_client
        @gitaly_ref_client ||= Gitlab::GitalyClient::RefService.new(self, vcs: 'hg')
      end

      def gitaly_commit_client
        @gitaly_commit_client ||= Gitlab::GitalyClient::CommitService.new(self, vcs: 'hg')
      end
    end
  end
end
